@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,12 @@
 
 import os
 import time
-from gettext import gettext as _
+from swift import gettext_ as _
 from random import random
 
 import swift.common.db
 from swift.account import server as account_server
-from swift.common.db import AccountBroker
+from swift.account.backend import AccountBroker
 from swift.common.utils import get_logger, audit_location_generator, \
     config_true_value, dump_recon_cache, ratelimit_sleep
 from swift.common.daemon import Daemon
@@ -60,9 +60,8 @@ class AccountAuditor(Daemon):
                                    '%(passed)s passed audit,'
                                    '%(failed)s failed audit'),
                                  {'time': time.ctime(reported),
-                                 'passed': self.account_passes,
-                                 'failed': self.account_failures})
-                self.account_audit(path)
+                                  'passed': self.account_passes,
+                                  'failed': self.account_failures})
                 dump_recon_cache({'account_audits_since': reported,
                                   'account_audits_passed': self.account_passes,
                                   'account_audits_failed':
@@ -119,10 +118,10 @@ class AccountAuditor(Daemon):
                 broker.get_info()
                 self.logger.increment('passes')
                 self.account_passes += 1
-                self.logger.debug(_('Audit passed for %s') % broker.db_file)
+                self.logger.debug(_('Audit passed for %s') % broker)
         except (Exception, Timeout):
             self.logger.increment('failures')
             self.account_failures += 1
             self.logger.exception(_('ERROR Could not get account info %s'),
-                                  (broker.db_file))
+                                  path)
         self.logger.timing_since('timing', start_time)
