@@ -48,6 +48,7 @@ class AccountController(object):
 
     def __init__(self, conf, logger=None):
         self.logger = logger or get_logger(conf, log_route='account-server')
+        self.log_requests = config_true_value(conf.get('log_requests', 'true'))
         self.root = conf.get('devices', '/srv/node')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
         replication_server = conf.get('replication_server', None)
@@ -296,19 +297,20 @@ class AccountController(object):
         if res.headers.get('x-container-timestamp') is not None:
             additional_info += 'x-container-timestamp: %s' % \
                 res.headers['x-container-timestamp']
-        log_message = '%s - - [%s] "%s %s" %s %s "%s" "%s" "%s" %s "%s"' % (
-            req.remote_addr,
-            time.strftime('%d/%b/%Y:%H:%M:%S +0000', time.gmtime()),
-            req.method, req.path,
-            res.status.split()[0], res.content_length or '-',
-            req.headers.get('x-trans-id', '-'),
-            req.referer or '-', req.user_agent or '-',
-            trans_time,
-            additional_info)
-        if req.method.upper() == 'REPLICATE':
-            self.logger.debug(log_message)
-        else:
-            self.logger.info(log_message)
+        if self.log_requests:
+            log_msg = '%s - - [%s] "%s %s" %s %s "%s" "%s" "%s" %s "%s"' % (
+                req.remote_addr,
+                time.strftime('%d/%b/%Y:%H:%M:%S +0000', time.gmtime()),
+                req.method, req.path,
+                res.status.split()[0], res.content_length or '-',
+                req.headers.get('x-trans-id', '-'),
+                req.referer or '-', req.user_agent or '-',
+                trans_time,
+                additional_info)
+            if req.method.upper() == 'REPLICATE':
+                self.logger.debug(log_msg)
+            else:
+                self.logger.info(log_msg)
         return res(env, start_response)
 
 
