@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
 import os
 import random
 import math
@@ -22,7 +21,7 @@ import shutil
 import uuid
 import errno
 import re
-from gettext import gettext as _
+from swift import gettext_ as _
 
 from eventlet import GreenPool, sleep, Timeout
 from eventlet.green import subprocess
@@ -32,7 +31,7 @@ import swift.common.db
 from swift.common.direct_client import quote
 from swift.common.utils import get_logger, whataremyips, storage_directory, \
     renamer, mkdirs, lock_parent_directory, config_true_value, \
-    unlink_older_than, dump_recon_cache, rsync_ip
+    unlink_older_than, dump_recon_cache, rsync_ip, ismount
 from swift.common import ring
 from swift.common.http import HTTP_NOT_FOUND, HTTP_INSUFFICIENT_STORAGE
 from swift.common.bufferedhttp import BufferedHTTPConnection
@@ -60,7 +59,7 @@ def quarantine_db(object_file, server_type):
                      server_type + 's', os.path.basename(object_dir)))
     try:
         renamer(object_dir, quarantine_dir)
-    except OSError, e:
+    except OSError as e:
         if e.errno not in (errno.EEXIST, errno.ENOTEMPTY):
             raise
         quarantine_dir = "%s-%s" % (quarantine_dir, uuid.uuid4().hex)
@@ -127,7 +126,7 @@ class ReplConnection(BufferedHTTPConnection):
 
         :param args: list of json-encodable objects
 
-        :returns: httplib response object
+        :returns: bufferedhttp response object
         """
         try:
             body = simplejson.dumps(args)
@@ -296,8 +295,8 @@ class Replicator(Daemon):
         if objects:
             self.logger.debug(_(
                 'Synchronization for %s has fallen more than '
-                '%s rows behind; moving on and will try again next pass.') %
-                (broker.db_file, self.max_diffs * self.per_diff))
+                '%s rows behind; moving on and will try again next pass.'),
+                broker, self.max_diffs * self.per_diff)
             self.stats['diff_capped'] += 1
             self.logger.increment('diff_caps')
         else:
@@ -430,7 +429,7 @@ class Replicator(Daemon):
                 self.logger.error(
                     'Found %s for %s when it should be on partition %s; will '
                     'replicate out and remove.' % (object_file, name, bpart))
-        except (Exception, Timeout), e:
+        except (Exception, Timeout) as e:
             if 'no such table' in str(e):
                 self.logger.error(_('Quarantining DB %s'), object_file)
                 quarantine_db(broker.db_file, broker.db_type)
@@ -495,7 +494,7 @@ class Replicator(Daemon):
             shutil.rmtree(hash_dir, True)
         try:
             os.rmdir(suf_dir)
-        except OSError, err:
+        except OSError as err:
             if err.errno not in (errno.ENOENT, errno.ENOTEMPTY):
                 self.logger.exception(
                     _('ERROR while trying to clean up %s') % suf_dir)
@@ -529,7 +528,7 @@ class Replicator(Daemon):
         for node in self.ring.devs:
             if (node and node['replication_ip'] in ips and
                     node['replication_port'] == self.port):
-                if self.mount_check and not os.path.ismount(
+                if self.mount_check and not ismount(
                         os.path.join(self.root, node['device'])):
                     self.logger.warn(
                         _('Skipping %(device)s as it is not mounted') % node)
@@ -580,8 +579,7 @@ class ReplicatorRpc(object):
             return HTTPBadRequest(body='Invalid object type')
         op = args.pop(0)
         drive, partition, hsh = replicate_args
-        if self.mount_check and \
-                not os.path.ismount(os.path.join(self.root, drive)):
+        if self.mount_check and not ismount(os.path.join(self.root, drive)):
             return Response(status='507 %s is not mounted' % drive)
         db_file = os.path.join(self.root, drive,
                                storage_directory(self.datadir, partition, hsh),
@@ -604,9 +602,9 @@ class ReplicatorRpc(object):
         timemark = time.time()
         try:
             info = broker.get_replication_info()
-        except (Exception, Timeout), e:
+        except (Exception, Timeout) as e:
             if 'no such table' in str(e):
-                self.logger.error(_("Quarantining DB %s") % broker.db_file)
+                self.logger.error(_("Quarantining DB %s"), broker)
                 quarantine_db(broker.db_file, broker.db_type)
                 return HTTPNotFound()
             raise
