@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,15 +14,17 @@
 # limitations under the License.
 
 from eventlet import sleep, Timeout
+from eventlet.green import httplib, socket, urllib2
 import json
 from paste.deploy import loadapp
 import struct
 from sys import exc_info
-from urllib import quote
 import zlib
-from gettext import gettext as _
+from swift import gettext_ as _
+import urlparse
 from zlib import compressobj
 
+from swift.common.utils import quote
 from swift.common.http import HTTP_NOT_FOUND
 from swift.common.swob import Request
 
@@ -36,7 +38,7 @@ class UnexpectedResponse(Exception):
     """
 
     def __init__(self, message, resp):
-        super(UnexpectedResponse, self).__init__(self, message)
+        super(UnexpectedResponse, self).__init__(message)
         self.resp = resp
 
 
@@ -56,14 +58,23 @@ class CompressingFileReader(object):
 
     def __init__(self, file_obj, compresslevel=9, chunk_size=4096):
         self._f = file_obj
+        self.compresslevel = compresslevel
+        self.chunk_size = chunk_size
+        self.set_initial_state()
+
+    def set_initial_state(self):
+        """
+        Sets the object to the state needed for the first read.
+        """
+
+        self._f.seek(0)
         self._compressor = compressobj(
-            compresslevel, zlib.DEFLATED, -zlib.MAX_WBITS, zlib.DEF_MEM_LEVEL,
-            0)
+            self.compresslevel, zlib.DEFLATED, -zlib.MAX_WBITS,
+            zlib.DEF_MEM_LEVEL, 0)
         self.done = False
         self.first = True
         self.crc32 = 0
         self.total_size = 0
-        self.chunk_size = chunk_size
 
     def read(self, *a, **kw):
         """
@@ -105,6 +116,11 @@ class CompressingFileReader(object):
             return chunk
         raise StopIteration
 
+    def seek(self, offset, whence=0):
+        if not (offset == 0 and whence == 0):
+            raise NotImplementedError('Seek implemented on offset 0 only')
+        self.set_initial_state()
+
 
 class InternalClient(object):
     """
@@ -163,7 +179,7 @@ class InternalClient(object):
             sleep(2 ** (attempt + 1))
         if resp:
             raise UnexpectedResponse(
-                _('Unexpected response: %s' % (resp.status,)), resp)
+                _('Unexpected response: %s') % resp.status, resp)
         if exc_type:
             # To make pep8 tool happy, in place of raise t, v, tb:
             raise exc_type(*exc_value.args), None, exc_traceback
@@ -219,10 +235,7 @@ class InternalClient(object):
         :raises Exception: Exception is raised when code fails in an
                            unexpected way.
         """
-        if isinstance(marker, unicode):
-            marker = marker.encode('utf8')
-        if isinstance(end_marker, unicode):
-            end_marker = end_marker.encode('utf8')
+
         while True:
             resp = self.make_request(
                 'GET', '%s?format=json&marker=%s&end_marker=%s' %
@@ -250,15 +263,6 @@ class InternalClient(object):
                             not.
         """
 
-        if isinstance(account, unicode):
-            account = account.encode('utf-8')
-
-        if isinstance(container, unicode):
-            container = container.encode('utf-8')
-
-        if isinstance(obj, unicode):
-            obj = obj.encode('utf-8')
-
         path = '/v1/%s' % quote(account)
         if container:
             path += '/%s' % quote(container)
@@ -403,12 +407,12 @@ class InternalClient(object):
         :param account: The container's account.
         :param container: Container to check.
 
-        :returns : True if container exists, false otherwise.
-
         :raises UnexpectedResponse: Exception raised when requests fail
                                     to get a response with an acceptable status
         :raises Exception: Exception is raised when code fails in an
                            unexpected way.
+
+        :returns : True if container exists, false otherwise.
         """
 
         path = self.make_path(account, container)
@@ -673,3 +677,108 @@ class InternalClient(object):
         headers['Transfer-Encoding'] = 'chunked'
         path = self.make_path(account, container, obj)
         self.make_request('PUT', path, headers, (2,), fobj)
+
+
+def get_auth(url, user, key, auth_version='1.0', **kwargs):
+    if auth_version != '1.0':
+        exit('ERROR: swiftclient missing, only auth v1.0 supported')
+    req = urllib2.Request(url)
+    req.add_header('X-Auth-User', user)
+    req.add_header('X-Auth-Key', key)
+    conn = urllib2.urlopen(req)
+    headers = conn.info()
+    return (
+        headers.getheader('X-Storage-Url'),
+        headers.getheader('X-Auth-Token'))
+
+
+class SimpleClient(object):
+    """
+    Simple client that is used in bin/swift-dispersion-* and container sync
+    """
+    def __init__(self, url=None, token=None, starting_backoff=1,
+                 max_backoff=5, retries=5):
+        self.url = url
+        self.token = token
+        self.attempts = 0
+        self.starting_backoff = starting_backoff
+        self.max_backoff = max_backoff
+        self.retries = retries
+
+    def base_request(self, method, container=None, name=None, prefix=None,
+                     headers={}, proxy=None, contents=None, full_listing=None):
+        # Common request method
+        url = self.url
+
+        if self.token:
+            headers['X-Auth-Token'] = self.token
+
+        if container:
+            url = '%s/%s' % (url.rstrip('/'), quote(container))
+
+        if name:
+            url = '%s/%s' % (url.rstrip('/'), quote(name))
+
+        url += '?format=json'
+
+        if prefix:
+            url += '&prefix=%s' % prefix
+
+        if proxy:
+            proxy = urlparse.urlparse(proxy)
+            proxy = urllib2.ProxyHandler({proxy.scheme: proxy.netloc})
+            opener = urllib2.build_opener(proxy)
+            urllib2.install_opener(opener)
+
+        req = urllib2.Request(url, headers=headers, data=contents)
+        req.get_method = lambda: method
+        urllib2.urlopen(req)
+        conn = urllib2.urlopen(req)
+        body = conn.read()
+        try:
+            body_data = json.loads(body)
+        except ValueError:
+            body_data = None
+        return [None, body_data]
+
+    def retry_request(self, method, **kwargs):
+        self.attempts = 0
+        backoff = self.starting_backoff
+        while self.attempts <= self.retries:
+            self.attempts += 1
+            try:
+                return self.base_request(method, **kwargs)
+            except (socket.error, httplib.HTTPException, urllib2.URLError):
+                if self.attempts > self.retries:
+                    raise
+            sleep(backoff)
+            backoff = min(backoff * 2, self.max_backoff)
+
+    def get_account(self, *args, **kwargs):
+        # Used in swift-dispertion-populate
+        return self.retry_request('GET', **kwargs)
+
+    def put_container(self, container, **kwargs):
+        # Used in swift-dispertion-populate
+        return self.retry_request('PUT', container=container, **kwargs)
+
+    def get_container(self, container, **kwargs):
+        # Used in swift-dispertion-populate
+        return self.retry_request('GET', container=container, **kwargs)
+
+    def put_object(self, container, name, contents, **kwargs):
+        # Used in swift-dispertion-populate
+        return self.retry_request('PUT', container=container, name=name,
+                                  contents=contents.read(), **kwargs)
+
+
+def put_object(url, **kwargs):
+    """For usage with container sync """
+    client = SimpleClient(url=url)
+    client.retry_request('PUT', **kwargs)
+
+
+def delete_object(url, **kwargs):
+    """For usage with container sync """
+    client = SimpleClient(url=url)
+    client.retry_request('DELETE', **kwargs)
