@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,29 +15,27 @@
 
 import os
 import time
-from gettext import gettext as _
-
+from swift import gettext_ as _
+from contextlib import closing
 from eventlet import Timeout
 
 from swift.obj import diskfile
-from swift.obj import server as object_server
-from swift.common.utils import get_logger, audit_location_generator, \
-    ratelimit_sleep, config_true_value, dump_recon_cache, list_from_csv, json
-from swift.common.exceptions import AuditException, DiskFileError, \
-    DiskFileNotExist
+from swift.common.utils import get_logger, ratelimit_sleep, dump_recon_cache, \
+    list_from_csv, json
+from swift.common.exceptions import DiskFileQuarantined, DiskFileNotExist
 from swift.common.daemon import Daemon
 
 SLEEP_BETWEEN_AUDITS = 30
 
 
 class AuditorWorker(object):
-    """Walk through file system to audit object"""
+    """Walk through file system to audit objects"""
 
     def __init__(self, conf, logger, zero_byte_only_at_fps=0):
         self.conf = conf
         self.logger = logger
         self.devices = conf.get('devices', '/srv/node')
-        self.mount_check = config_true_value(conf.get('mount_check', 'true'))
+        self.diskfile_mgr = diskfile.DiskFileManager(conf, self.logger)
         self.max_files_per_second = float(conf.get('files_per_second', 20))
         self.max_bytes_per_second = float(conf.get('bytes_per_second',
                                                    10000000))
@@ -64,21 +62,18 @@ class AuditorWorker(object):
             [(s, 0) for s in self.stats_sizes + ['OVER']])
 
     def audit_all_objects(self, mode='once'):
-        self.logger.info(_('Begin object audit "%s" mode (%s)' %
-                           (mode, self.auditor_type)))
+        self.logger.info(_('Begin object audit "%s" mode (%s)') %
+                         (mode, self.auditor_type))
         begin = reported = time.time()
         self.total_bytes_processed = 0
         self.total_files_processed = 0
         total_quarantines = 0
         total_errors = 0
         time_auditing = 0
-        all_locs = audit_location_generator(self.devices,
-                                            object_server.DATADIR, '.data',
-                                            mount_check=self.mount_check,
-                                            logger=self.logger)
-        for path, device, partition in all_locs:
+        all_locs = self.diskfile_mgr.object_audit_location_generator()
+        for location in all_locs:
             loop_time = time.time()
-            self.object_audit(path, device, partition)
+            self.failsafe_object_audit(location)
             self.logger.timing_since('timing', loop_time)
             self.files_running_time = ratelimit_sleep(
                 self.files_running_time, self.max_files_per_second)
@@ -122,11 +117,12 @@ class AuditorWorker(object):
         self.logger.info(_(
             'Object audit (%(type)s) "%(mode)s" mode '
             'completed: %(elapsed).02fs. Total quarantined: %(quars)d, '
-            'Total errors: %(errors)d, Total files/sec: %(frate).2f , '
+            'Total errors: %(errors)d, Total files/sec: %(frate).2f, '
             'Total bytes/sec: %(brate).2f, Auditing time: %(audit).2f, '
             'Rate: %(audit_rate).2f') % {
                 'type': self.auditor_type, 'mode': mode, 'elapsed': elapsed,
-                'quars': total_quarantines, 'errors': total_errors,
+                'quars': total_quarantines + self.quarantines,
+                'errors': total_errors + self.errors,
                 'frate': self.total_files_processed / elapsed,
                 'brate': self.total_bytes_processed / elapsed,
                 'audit': time_auditing, 'audit_rate': time_auditing / elapsed})
@@ -151,64 +147,54 @@ class AuditorWorker(object):
         else:
             self.stats_buckets["OVER"] += 1
 
-    def object_audit(self, path, device, partition):
+    def failsafe_object_audit(self, location):
         """
-        Audits the given object path.
-
-        :param path: a path to an object
-        :param device: the device the path is on
-        :param partition: the partition the path is on
+        Entrypoint to object_audit, with a failsafe generic exception handler.
         """
         try:
-            try:
-                name = diskfile.read_metadata(path)['name']
-            except (Exception, Timeout) as exc:
-                raise AuditException('Error when reading metadata: %s' % exc)
-            _junk, account, container, obj = name.split('/', 3)
-            df = diskfile.DiskFile(self.devices, device, partition,
-                                   account, container, obj, self.logger,
-                                   keep_data_fp=True)
-            try:
-                try:
-                    obj_size = df.get_data_file_size()
-                except DiskFileNotExist:
-                    return
-                except DiskFileError as e:
-                    raise AuditException(str(e))
+            self.object_audit(location)
+        except (Exception, Timeout):
+            self.logger.increment('errors')
+            self.errors += 1
+            self.logger.exception(_('ERROR Trying to audit %s'), location)
+
+    def object_audit(self, location):
+        """
+        Audits the given object location.
+
+        :param location: an audit location
+                         (from diskfile.object_audit_location_generator)
+        """
+        def raise_dfq(msg):
+            raise DiskFileQuarantined(msg)
+
+        try:
+            df = self.diskfile_mgr.get_diskfile_from_audit_location(location)
+            with df.open():
+                metadata = df.get_metadata()
+                obj_size = int(metadata['Content-Length'])
                 if self.stats_sizes:
                     self.record_stats(obj_size)
                 if self.zero_byte_only_at_fps and obj_size:
                     self.passes += 1
                     return
-                for chunk in df:
+                reader = df.reader(_quarantine_hook=raise_dfq)
+            with closing(reader):
+                for chunk in reader:
+                    chunk_len = len(chunk)
                     self.bytes_running_time = ratelimit_sleep(
-                        self.bytes_running_time, self.max_bytes_per_second,
-                        incr_by=len(chunk))
-                    self.bytes_processed += len(chunk)
-                    self.total_bytes_processed += len(chunk)
-                df.close()
-                if df.quarantined_dir:
-                    self.quarantines += 1
-                    self.logger.error(
-                        _("ERROR Object %(path)s failed audit and will be "
-                          "quarantined: ETag and file's md5 do not match"),
-                        {'path': path})
-            finally:
-                df.close(verify_file=False)
-        except AuditException as err:
-            self.logger.increment('quarantines')
+                        self.bytes_running_time,
+                        self.max_bytes_per_second,
+                        incr_by=chunk_len)
+                    self.bytes_processed += chunk_len
+                    self.total_bytes_processed += chunk_len
+        except DiskFileNotExist:
+            return
+        except DiskFileQuarantined as err:
             self.quarantines += 1
-            self.logger.error(_('ERROR Object %(obj)s failed audit and will '
-                                'be quarantined: %(err)s'),
-                              {'obj': path, 'err': err})
-            diskfile.quarantine_renamer(
-                os.path.join(self.devices, device), path)
-            return
-        except (Exception, Timeout):
-            self.logger.increment('errors')
-            self.errors += 1
-            self.logger.exception(_('ERROR Trying to audit %s'), path)
-            return
+            self.logger.error(_('ERROR Object %(obj)s failed audit and was'
+                                ' quarantined: %(err)s'),
+                              {'obj': location, 'err': err})
         self.passes += 1
 
 
