@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
-
 import unittest
 import os
 import mock
@@ -126,10 +124,12 @@ def _create_test_ring(path):
     intended_part_shift = 30
     intended_reload_time = 15
     with closing(GzipFile(testgz, 'wb')) as f:
-        pickle.dump(ring.RingData(intended_replica2part2dev_id,
-            intended_devs, intended_part_shift),
+        pickle.dump(
+            ring.RingData(intended_replica2part2dev_id,
+                          intended_devs, intended_part_shift),
             f)
-    return ring.Ring(path, ring_name='object', reload_time=intended_reload_time)
+    return ring.Ring(path, ring_name='object',
+                     reload_time=intended_reload_time)
 
 
 class TestObjectReplicator(unittest.TestCase):
@@ -154,9 +154,10 @@ class TestObjectReplicator(unittest.TestCase):
         self.conf = dict(
             swift_dir=self.testdir, devices=self.devices, mount_check='false',
             timeout='300', stats_interval='1')
-        self.replicator = object_replicator.ObjectReplicator(
-            self.conf)
+        self.replicator = object_replicator.ObjectReplicator(self.conf)
         self.replicator.logger = FakeLogger()
+        self.df_mgr = diskfile.DiskFileManager(self.conf,
+                                               self.replicator.logger)
 
     def tearDown(self):
         rmtree(self.testdir, ignore_errors=1)
@@ -164,14 +165,13 @@ class TestObjectReplicator(unittest.TestCase):
     def test_run_once(self):
         replicator = object_replicator.ObjectReplicator(
             dict(swift_dir=self.testdir, devices=self.devices,
-                mount_check='false', timeout='300', stats_interval='1'))
+                 mount_check='false', timeout='300', stats_interval='1'))
         was_connector = object_replicator.http_connect
         object_replicator.http_connect = mock_http_connect(200)
         cur_part = '0'
-        df = diskfile.DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
-                               FakeLogger())
-        mkdirs(df.datadir)
-        f = open(os.path.join(df.datadir,
+        df = self.df_mgr.get_diskfile('sda', cur_part, 'a', 'c', 'o')
+        mkdirs(df._datadir)
+        f = open(os.path.join(df._datadir,
                               normalize_timestamp(time.time()) + '.data'),
                  'wb')
         f.write('1234567890')
@@ -182,7 +182,7 @@ class TestObjectReplicator(unittest.TestCase):
         process_arg_checker = []
         nodes = [node for node in
                  self.ring.get_part_nodes(int(cur_part))
-                     if node['ip'] not in _ips()]
+                 if node['ip'] not in _ips()]
         for node in nodes:
             rsync_mod = '%s::object/sda/objects/%s' % (node['ip'], cur_part)
             process_arg_checker.append(
@@ -234,7 +234,7 @@ class TestObjectReplicator(unittest.TestCase):
         for job in jobs:
             jobs_by_part[job['partition']] = job
         self.assertEquals(len(jobs_to_delete), 1)
-        self.assertTrue('1', jobs_to_delete[0]['partition'])
+        self.assertEquals('1', jobs_to_delete[0]['partition'])
         self.assertEquals(
             [node['id'] for node in jobs_by_part['0']['nodes']], [1, 2])
         self.assertEquals(
@@ -249,6 +249,12 @@ class TestObjectReplicator(unittest.TestCase):
             self.assertEquals(jobs_by_part[part]['path'],
                               os.path.join(self.objects, part))
 
+    def test_collect_jobs_handoffs_first(self):
+        self.replicator.handoffs_first = True
+        jobs = self.replicator.collect_jobs()
+        self.assertTrue(jobs[0]['delete'])
+        self.assertEquals('1', jobs[0]['partition'])
+
     def test_collect_jobs_removes_zbf(self):
         """
         After running xfs_repair, a partition directory could become a
@@ -289,18 +295,137 @@ class TestObjectReplicator(unittest.TestCase):
     def test_delete_partition(self):
         with mock.patch('swift.obj.replicator.http_connect',
                         mock_http_connect(200)):
-            df = diskfile.DiskFile(self.devices,
-                                   'sda', '0', 'a', 'c', 'o', FakeLogger())
-            mkdirs(df.datadir)
+            df = self.df_mgr.get_diskfile('sda', '1', 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            print df._datadir
+            f = open(os.path.join(df._datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, '1', data_dir)
             part_path = os.path.join(self.objects, '1')
             self.assertTrue(os.access(part_path, os.F_OK))
-            self.replicator.replicate()
+            nodes = [node for node in
+                     self.ring.get_part_nodes(1)
+                     if node['ip'] not in _ips()]
+            process_arg_checker = []
+            for node in nodes:
+                rsync_mod = '%s::object/sda/objects/%s' % (node['ip'], 1)
+                process_arg_checker.append(
+                    (0, '', ['rsync', whole_path_from, rsync_mod]))
+            with _mock_process(process_arg_checker):
+                self.replicator.replicate()
             self.assertFalse(os.access(part_path, os.F_OK))
 
+    def test_delete_partition_with_failures(self):
+        with mock.patch('swift.obj.replicator.http_connect',
+                        mock_http_connect(200)):
+            df = self.df_mgr.get_diskfile('sda', '1', 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            print df._datadir
+            f = open(os.path.join(df._datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, '1', data_dir)
+            part_path = os.path.join(self.objects, '1')
+            self.assertTrue(os.access(part_path, os.F_OK))
+            nodes = [node for node in
+                     self.ring.get_part_nodes(1)
+                     if node['ip'] not in _ips()]
+            process_arg_checker = []
+            for i, node in enumerate(nodes):
+                rsync_mod = '%s::object/sda/objects/%s' % (node['ip'], 1)
+                if i == 0:
+                    # force one of the rsync calls to fail
+                    ret_code = 1
+                else:
+                    ret_code = 0
+                process_arg_checker.append(
+                    (ret_code, '', ['rsync', whole_path_from, rsync_mod]))
+            with _mock_process(process_arg_checker):
+                self.replicator.replicate()
+            # The path should still exist
+            self.assertTrue(os.access(part_path, os.F_OK))
+
+    def test_delete_partition_with_handoff_delete(self):
+        with mock.patch('swift.obj.replicator.http_connect',
+                        mock_http_connect(200)):
+            self.replicator.handoff_delete = 2
+            df = self.df_mgr.get_diskfile('sda', '1', 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            print df._datadir
+            f = open(os.path.join(df._datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, '1', data_dir)
+            part_path = os.path.join(self.objects, '1')
+            self.assertTrue(os.access(part_path, os.F_OK))
+            nodes = [node for node in
+                     self.ring.get_part_nodes(1)
+                     if node['ip'] not in _ips()]
+            process_arg_checker = []
+            for i, node in enumerate(nodes):
+                rsync_mod = '%s::object/sda/objects/%s' % (node['ip'], 1)
+                if i == 0:
+                    # force one of the rsync calls to fail
+                    ret_code = 1
+                else:
+                    ret_code = 0
+                process_arg_checker.append(
+                    (ret_code, '', ['rsync', whole_path_from, rsync_mod]))
+            with _mock_process(process_arg_checker):
+                self.replicator.replicate()
+            self.assertFalse(os.access(part_path, os.F_OK))
+
+    def test_delete_partition_with_handoff_delete_failures(self):
+        with mock.patch('swift.obj.replicator.http_connect',
+                        mock_http_connect(200)):
+            self.replicator.handoff_delete = 2
+            df = self.df_mgr.get_diskfile('sda', '1', 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            print df._datadir
+            f = open(os.path.join(df._datadir,
+                                  normalize_timestamp(time.time()) + '.data'),
+                     'wb')
+            f.write('1234567890')
+            f.close()
+            ohash = hash_path('a', 'c', 'o')
+            data_dir = ohash[-3:]
+            whole_path_from = os.path.join(self.objects, '1', data_dir)
+            part_path = os.path.join(self.objects, '1')
+            self.assertTrue(os.access(part_path, os.F_OK))
+            nodes = [node for node in
+                     self.ring.get_part_nodes(1)
+                     if node['ip'] not in _ips()]
+            process_arg_checker = []
+            for i, node in enumerate(nodes):
+                rsync_mod = '%s::object/sda/objects/%s' % (node['ip'], 1)
+                if i in (0, 1):
+                    # force two of the rsync calls to fail
+                    ret_code = 1
+                else:
+                    ret_code = 0
+                process_arg_checker.append(
+                    (ret_code, '', ['rsync', whole_path_from, rsync_mod]))
+            with _mock_process(process_arg_checker):
+                self.replicator.replicate()
+            # The file should still exist
+            self.assertTrue(os.access(part_path, os.F_OK))
+
     def test_delete_partition_override_params(self):
-        df = diskfile.DiskFile(self.devices, 'sda', '0', 'a', 'c', 'o',
-                               FakeLogger())
-        mkdirs(df.datadir)
+        df = self.df_mgr.get_diskfile('sda', '0', 'a', 'c', 'o')
+        mkdirs(df._datadir)
         part_path = os.path.join(self.objects, '1')
         self.assertTrue(os.access(part_path, os.F_OK))
         self.replicator.replicate(override_devices=['sdb'])
@@ -314,17 +439,16 @@ class TestObjectReplicator(unittest.TestCase):
     def test_run_once_recover_from_failure(self):
         replicator = object_replicator.ObjectReplicator(
             dict(swift_dir=self.testdir, devices=self.devices,
-                mount_check='false', timeout='300', stats_interval='1'))
+                 mount_check='false', timeout='300', stats_interval='1'))
         was_connector = object_replicator.http_connect
         try:
             object_replicator.http_connect = mock_http_connect(200)
             # Write some files into '1' and run replicate- they should be moved
             # to the other partitoins and then node should get deleted.
             cur_part = '1'
-            df = diskfile.DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
-                                   FakeLogger())
-            mkdirs(df.datadir)
-            f = open(os.path.join(df.datadir,
+            df = self.df_mgr.get_diskfile('sda', cur_part, 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            f = open(os.path.join(df._datadir,
                                   normalize_timestamp(time.time()) + '.data'),
                      'wb')
             f.write('1234567890')
@@ -335,7 +459,7 @@ class TestObjectReplicator(unittest.TestCase):
             process_arg_checker = []
             nodes = [node for node in
                      self.ring.get_part_nodes(int(cur_part))
-                         if node['ip'] not in _ips()]
+                     if node['ip'] not in _ips()]
             for node in nodes:
                 rsync_mod = '%s::object/sda/objects/%s' % (node['ip'],
                                                            cur_part)
@@ -350,16 +474,16 @@ class TestObjectReplicator(unittest.TestCase):
             for i, result in [('0', True), ('1', False),
                               ('2', True), ('3', True)]:
                 self.assertEquals(os.access(
-                        os.path.join(self.objects,
-                                     i, diskfile.HASH_FILE),
-                        os.F_OK), result)
+                    os.path.join(self.objects,
+                                 i, diskfile.HASH_FILE),
+                    os.F_OK), result)
         finally:
             object_replicator.http_connect = was_connector
 
     def test_run_once_recover_from_timeout(self):
         replicator = object_replicator.ObjectReplicator(
             dict(swift_dir=self.testdir, devices=self.devices,
-                mount_check='false', timeout='300', stats_interval='1'))
+                 mount_check='false', timeout='300', stats_interval='1'))
         was_connector = object_replicator.http_connect
         was_get_hashes = object_replicator.get_hashes
         was_execute = tpool.execute
@@ -385,10 +509,9 @@ class TestObjectReplicator(unittest.TestCase):
             # Write some files into '1' and run replicate- they should be moved
             # to the other partitions and then node should get deleted.
             cur_part = '1'
-            df = diskfile.DiskFile(self.devices, 'sda', cur_part, 'a', 'c', 'o',
-                                   FakeLogger())
-            mkdirs(df.datadir)
-            f = open(os.path.join(df.datadir,
+            df = self.df_mgr.get_diskfile('sda', cur_part, 'a', 'c', 'o')
+            mkdirs(df._datadir)
+            f = open(os.path.join(df._datadir,
                                   normalize_timestamp(time.time()) + '.data'),
                      'wb')
             f.write('1234567890')
@@ -399,7 +522,7 @@ class TestObjectReplicator(unittest.TestCase):
             process_arg_checker = []
             nodes = [node for node in
                      self.ring.get_part_nodes(int(cur_part))
-                         if node['ip'] not in _ips()]
+                     if node['ip'] not in _ips()]
             for node in nodes:
                 rsync_mod = '%s::object/sda/objects/%s' % (node['ip'],
                                                            cur_part)
@@ -419,11 +542,21 @@ class TestObjectReplicator(unittest.TestCase):
 
     def test_run(self):
         with _mock_process([(0, '')] * 100):
-            self.replicator.replicate()
+            with mock.patch('swift.obj.replicator.http_connect',
+                            mock_http_connect(200)):
+                self.replicator.replicate()
 
     def test_run_withlog(self):
         with _mock_process([(0, "stuff in log")] * 100):
-            self.replicator.replicate()
+            with mock.patch('swift.obj.replicator.http_connect',
+                            mock_http_connect(200)):
+                self.replicator.replicate()
+
+    def test_sync_just_calls_sync_method(self):
+        self.replicator.sync_method = mock.MagicMock()
+        self.replicator.sync('node', 'job', 'suffixes')
+        self.replicator.sync_method.assert_called_once_with(
+            'node', 'job', 'suffixes')
 
     @mock.patch('swift.obj.replicator.tpool_reraise', autospec=True)
     @mock.patch('swift.obj.replicator.http_connect', autospec=True)
@@ -460,9 +593,9 @@ class TestObjectReplicator(unittest.TestCase):
             self.assertEquals(mock_http.call_count, len(self.ring._devs) - 1)
             reqs = []
             for node in job['nodes']:
-               reqs.append(mock.call(node['ip'], node['port'], node['device'],
-                                     job['partition'], 'REPLICATE', '',
-                                     headers=self.headers))
+                reqs.append(mock.call(node['ip'], node['port'], node['device'],
+                                      job['partition'], 'REPLICATE', '',
+                                      headers=self.headers))
             if job['partition'] == '0':
                 self.assertEquals(self.replicator.suffix_hash, 0)
             mock_http.assert_has_calls(reqs, any_order=True)
@@ -509,13 +642,13 @@ class TestObjectReplicator(unittest.TestCase):
             self.assertEquals(self.replicator.suffix_count, 0)
             mock_logger.reset_mock()
 
-        # Check seccesfull http_connect and rsync for local node
+        # Check successful http_connect and sync for local node
         mock_tpool_reraise.return_value = (1, {'a83': 'ba47fd314242ec8c'
                                                       '7efb91f5d57336e4'})
         resp.read.return_value = pickle.dumps({'a83': 'c130a2c17ed45102a'
                                                       'ada0f4eee69494ff'})
         set_default(self)
-        self.replicator.rsync = fake_func = mock.MagicMock()
+        self.replicator.sync = fake_func = mock.MagicMock()
         self.replicator.update(local_job)
         reqs = []
         for node in local_job['nodes']:
@@ -538,14 +671,14 @@ class TestObjectReplicator(unittest.TestCase):
         self.replicator.update(repl_job)
         reqs = []
         for node in repl_job['nodes']:
-           reqs.append(mock.call(node['replication_ip'],
-                                 node['replication_port'], node['device'],
-                                 repl_job['partition'], 'REPLICATE',
-                                 '', headers=self.headers))
-           reqs.append(mock.call(node['replication_ip'],
-                                 node['replication_port'], node['device'],
-                                 repl_job['partition'], 'REPLICATE',
-                                 '/a83', headers=self.headers))
+            reqs.append(mock.call(node['replication_ip'],
+                                  node['replication_port'], node['device'],
+                                  repl_job['partition'], 'REPLICATE',
+                                  '', headers=self.headers))
+            reqs.append(mock.call(node['replication_ip'],
+                                  node['replication_port'], node['device'],
+                                  repl_job['partition'], 'REPLICATE',
+                                  '/a83', headers=self.headers))
         mock_http.assert_has_calls(reqs, any_order=True)
 
 
